@@ -1,5 +1,3 @@
-# -*- coding: utf-8; -*-
-#
 # test/helper.py
 # Part of ‘dput’, a Debian package upload toolkit.
 #
@@ -9,34 +7,10 @@
 
 """ Helper functionality for Dput test suite. """
 
-from __future__ import (absolute_import, unicode_literals)
-
-import sys
-
-if sys.version_info >= (3, 3):
-    import builtins
-    import collections.abc as collections_abc
-    import configparser
-    from io import StringIO as StringIO
-    import unittest
-    import unittest.mock as mock
-elif sys.version_info >= (3, 0):
-    raise RuntimeError("Python 3 earlier than 3.3 is not supported.")
-elif sys.version_info >= (2, 7):
-    # Python 2 standard library.
-    import __builtin__ as builtins
-    import collections as collections_abc
-    import ConfigParser as configparser
-    from StringIO import StringIO as BaseStringIO
-    # Third-party backport of Python 3 unittest improvements.
-    import unittest2 as unittest
-    # Third-party mock library.
-    import mock
-else:
-    raise RuntimeError("Python earlier than 2.7 is not supported.")
-
 import base64
+import builtins
 import collections
+import collections.abc
 import errno
 import functools
 import io
@@ -48,13 +22,13 @@ import shlex
 import shutil
 import signal
 import subprocess
+import sys
 import tempfile
 import time
+import unittest.mock
 import weakref
 
 
-__metaclass__ = type
-
 try:
     # Python 2 types.
     basestring
@@ -73,51 +47,34 @@ def make_unique_slug(testcase):
     result = text[-30:]
     return result
 
-
-try:
-    StringIO
-except NameError:
-    # We don't yet have the StringIO we want. Create it.
 
-    class StringIO(BaseStringIO, object):
-        """ StringIO with a context manager. """
-
-        def __enter__(self):
-            return self
-
-        def __exit__(self, *args):
-            self.close()
-            return False
-
-        def readable(self):
-            return True
-
-        def writable(self):
-            return True
-
-        def seekable(self):
-            return True
+def make_fake_fqdn(testcase):
+    """ Make a fake Fully-Qualified Domain Name (FQDN). """
+    slug = make_unique_slug(testcase)
+    fqdn = "{}.example.net".format(slug)
+    return fqdn
 
 
 def patch_stdout(testcase):
     """ Patch `sys.stdout` for the specified test case. """
-    patcher = mock.patch.object(
-            sys, "stdout", wraps=StringIO())
+    patcher = unittest.mock.patch.object(
+            sys, "stdout", wraps=io.StringIO())
     patcher.start()
     testcase.addCleanup(patcher.stop)
 
 
 def patch_stderr(testcase):
     """ Patch `sys.stderr` for the specified test case. """
-    patcher = mock.patch.object(
-            sys, "stderr", wraps=StringIO())
+    patcher = unittest.mock.patch.object(
+            sys, "stderr", wraps=io.StringIO())
     patcher.start()
     testcase.addCleanup(patcher.stop)
 
 
 def patch_signal_signal(testcase):
     """ Patch `signal.signal` for the specified test case. """
-    func_patcher = mock.patch.object(signal, "signal", autospec=True)
+    func_patcher = unittest.mock.patch.object(
+            signal, "signal", autospec=True)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
@@ -133,7 +90,7 @@ EXIT_STATUS_COMMAND_NOT_FOUND = 127
 
 def patch_sys_exit(testcase):
     """ Patch `sys.exit` for the specified test case. """
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             sys, "exit", autospec=True,
             side_effect=FakeSystemExit())
     func_patcher.start()
@@ -146,7 +103,7 @@ def patch_sys_argv(testcase):
         testcase.progname = make_unique_slug(testcase)
     if not hasattr(testcase, 'sys_argv'):
         testcase.sys_argv = [testcase.progname]
-    patcher = mock.patch.object(
+    patcher = unittest.mock.patch.object(
             sys, "argv",
             new=list(testcase.sys_argv))
     patcher.start()
@@ -176,7 +133,7 @@ def patch_time_time(testcase, values=None):
         while True:
             yield next(values)
 
-    func_patcher = mock.patch.object(time, "time", autospec=True)
+    func_patcher = unittest.mock.patch.object(time, "time", autospec=True)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
@@ -187,14 +144,15 @@ def patch_os_environ(testcase):
     """ Patch the `os.environ` mapping. """
     if not hasattr(testcase, 'os_environ'):
         testcase.os_environ = {}
-    patcher = mock.patch.object(os, "environ", new=testcase.os_environ)
+    patcher = unittest.mock.patch.object(
+            os, "environ", new=testcase.os_environ)
     patcher.start()
     testcase.addCleanup(patcher.stop)
 
 
 def patch_os_getpid(testcase):
     """ Patch `os.getpid` for the specified test case. """
-    func_patcher = mock.patch.object(os, "getpid", autospec=True)
+    func_patcher = unittest.mock.patch.object(os, "getpid", autospec=True)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
@@ -203,7 +161,7 @@ def patch_os_getuid(testcase):
     """ Patch the `os.getuid` function. """
     if not hasattr(testcase, 'os_getuid_return_value'):
         testcase.os_getuid_return_value = testcase.getUniqueInteger()
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "getuid", autospec=True,
             return_value=testcase.os_getuid_return_value)
     func_patcher.start()
@@ -230,7 +188,7 @@ def patch_pwd_getpwuid(testcase):
         pwent = pwd.struct_passwd(testcase.pwd_getpwuid_return_value)
     else:
         pwent = testcase.pwd_getpwuid_return_value
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             pwd, "getpwuid", autospec=True,
             return_value=pwent)
     func_patcher.start()
@@ -256,7 +214,7 @@ def patch_os_path_exists(testcase):
             result = orig_os_path_exists(path)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os.path, "exists", autospec=True,
             side_effect=fake_os_path_exists)
     func_patcher.start()
@@ -282,7 +240,7 @@ def patch_os_access(testcase):
             result = orig_os_access(path, mode)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "access", autospec=True,
             side_effect=fake_os_access)
     func_patcher.start()
@@ -318,7 +276,7 @@ def patch_os_stat(testcase):
             result = orig_os_stat(path)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "stat", autospec=True,
             side_effect=fake_os_stat)
     func_patcher.start()
@@ -344,12 +302,43 @@ def patch_os_lstat(testcase):
             result = orig_os_lstat(path)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "lstat", autospec=True,
             side_effect=fake_os_lstat)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
+
+def patch_builtins_open(testcase):
+    """ Patch `builtins.open` behaviour for this test case.
+
+        :param testcase: The `TestCase` instance for binding to the patch.
+        :return: None.
+
+        When the patched function is called, the registry of
+        `FileDouble` instances for this test case will be used to get
+        the instance for the path specified.
+
+        """
+    orig_open = builtins.open
+
+    def fake_open(path, mode='rt', buffering=-1):
+        registry = FileDouble.get_registry_for_testcase(testcase)
+        if path in registry:
+            file_double = registry[path]
+            result = file_double.builtins_open_scenario.call_hook(
+                    mode, buffering)
+        else:
+            result = orig_open(path, mode, buffering)
+        return result
+
+    mock_open = mock.mock_open()
+    mock_open.side_effect = fake_open
+
+    func_patcher = mock.patch.object(builtins, "open", new=mock_open)
+    func_patcher.start()
+    testcase.addCleanup(func_patcher.stop)
+
 
 def patch_os_unlink(testcase):
     """ Patch `os.unlink` behaviour for this test case.
@@ -370,7 +359,7 @@ def patch_os_unlink(testcase):
             result = orig_os_unlink(path)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "unlink", autospec=True,
             side_effect=fake_os_unlink)
     func_patcher.start()
@@ -396,7 +385,7 @@ def patch_os_rmdir(testcase):
             result = orig_os_rmdir(path)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "rmdir", autospec=True,
             side_effect=fake_os_rmdir)
     func_patcher.start()
@@ -422,7 +411,7 @@ def patch_shutil_rmtree(testcase):
             result = orig_shutil_rmtree(path)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             shutil, "rmtree", autospec=True,
             side_effect=fake_shutil_rmtree)
     func_patcher.start()
@@ -439,7 +428,8 @@ def patch_tempfile_mkdtemp(testcase):
     double.set_os_rmdir_scenario('okay')
     double.register_for_testcase(testcase)
 
-    func_patcher = mock.patch.object(tempfile, "mkdtemp", autospec=True)
+    func_patcher = unittest.mock.patch.object(
+            tempfile, "mkdtemp", autospec=True)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
@@ -478,19 +468,19 @@ except NameError:
         def __init__(self, *args, **kwargs):
             (args, kwargs) = _ensure_ioerror_args(
                     args, kwargs, errno_value=errno.ENOENT)
-            super(FileNotFoundError, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
 
     class FileExistsError(IOError):
         def __init__(self, *args, **kwargs):
             (args, kwargs) = _ensure_ioerror_args(
                     args, kwargs, errno_value=errno.EEXIST)
-            super(FileExistsError, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
 
     class PermissionError(IOError):
         def __init__(self, *args, **kwargs):
             (args, kwargs) = _ensure_ioerror_args(
                     args, kwargs, errno_value=errno.EPERM)
-            super(PermissionError, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
 
 
 def make_fake_file_scenarios(path=None):
@@ -510,9 +500,9 @@ def make_fake_file_scenarios(path=None):
     else:
         file_path = path
 
-    fake_file_empty = StringIO()
-    fake_file_minimal = StringIO("Lorem ipsum.")
-    fake_file_large = StringIO("\n".join(
+    fake_file_empty = io.StringIO()
+    fake_file_minimal = io.StringIO("Lorem ipsum.")
+    fake_file_large = io.StringIO("\n".join(
             "ABCDEFGH" * 100
             for __ in range(1000)))
 
@@ -608,10 +598,10 @@ def setup_file_double_behaviour(testcase, doubles=None):
             result = orig_open(path, mode, buffering)
         return result
 
-    mock_open = mock.mock_open()
+    mock_open = unittest.mock.mock_open()
     mock_open.side_effect = fake_open
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             builtins, "open", new=mock_open)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
@@ -850,7 +840,7 @@ class TestDoubleWithRegistry:
     function_scenario_params_by_class = NotImplemented
 
     def __new__(cls, *args, **kwargs):
-        superclass = super(TestDoubleWithRegistry, cls)
+        superclass = super()
         if superclass.__new__ is object.__new__:
             # The ‘object’ implementation complains about extra arguments.
             instance = superclass.__new__(cls)
@@ -861,7 +851,7 @@ class TestDoubleWithRegistry:
         return instance
 
     def __init__(self, *args, **kwargs):
-        super(TestDoubleWithRegistry, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._set_method_per_scenario()
 
     def _make_set_scenario_method(self, scenario_class, params):
@@ -920,8 +910,8 @@ class TestDoubleWithRegistry:
 
 
 def copy_fake_file(fake_file):
-    """ Make a copy of the StringIO instance. """
-    fake_file_type = StringIO
+    """ Make a copy of the `StringIO` instance. """
+    fake_file_type = io.StringIO
     content = ""
     if fake_file is not None:
         fake_file_type = type(fake_file)
@@ -982,7 +972,7 @@ class FileDouble(TestDoubleWithRegistry):
 
         self._set_stat_result()
 
-        super(FileDouble, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _set_stat_result(self):
         """ Set the `os.stat` result for this file. """
@@ -1022,11 +1012,11 @@ class os_popen_scenario(TestDoubleFunctionScenario):
         return result
 
     def _hook_failure(self, argv, mode, buffering):
-        result = StringIO()
+        result = io.StringIO()
         return result
 
     def _hook_not_found(self, argv, mode, buffering):
-        result = StringIO()
+        result = io.StringIO()
         return result
 
 
@@ -1164,7 +1154,7 @@ def patch_subprocess_popen(testcase):
             result = orig_subprocess_popen(args, *posargs, **kwargs)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             subprocess, "Popen", autospec=True,
             side_effect=fake_subprocess_popen)
     func_patcher.start()
@@ -1214,7 +1204,7 @@ def patch_subprocess_call(testcase):
             result = orig_subprocess_call(command, *posargs, **kwargs)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             subprocess, "call", autospec=True,
             side_effect=fake_subprocess_call)
     func_patcher.start()
@@ -1265,22 +1255,22 @@ def patch_subprocess_check_call(testcase):
             result = orig_subprocess_check_call(command, *posargs, **kwargs)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             subprocess, "check_call", autospec=True,
             side_effect=fake_subprocess_check_call)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
 
-class SubprocessDoubleRegistry(collections_abc.MutableMapping):
+class SubprocessDoubleRegistry(collections.abc.MutableMapping):
     """ Registry of `SubprocessDouble` instances by `argv`. """
 
     def __init__(self, *args, **kwargs):
         items = []
         if args:
-            if isinstance(args[0], collections_abc.Mapping):
+            if isinstance(args[0], collections.abc.Mapping):
                 items = args[0].items()
-            if isinstance(args[0], collections_abc.Iterable):
+            if isinstance(args[0], collections.abc.Iterable):
                 items = args[0]
         self._mapping = dict(items)
 
@@ -1292,7 +1282,7 @@ class SubprocessDoubleRegistry(collections_abc.MutableMapping):
     def _match_argv(self, argv):
         """ Match the specified `argv` with our registered keys. """
         match = None
-        if not isinstance(argv, collections_abc.Sequence):
+        if not isinstance(argv, collections.abc.Sequence):
             return match
         candidates = iter(self._mapping)
         while match is None:
@@ -1418,7 +1408,7 @@ class SubprocessDouble(TestDoubleWithRegistry):
             stream_double_name = '{name}_double'.format(name=stream_name)
             setattr(self, stream_double_name, file_double)
 
-        super(SubprocessDouble, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def set_popen_double(self):
         """ Set the `PopenDouble` for this instance. """
@@ -1605,7 +1595,7 @@ def patch_os_popen(testcase):
             result = orig_os_popen(cmd, mode, buffering)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "popen", autospec=True,
             side_effect=fake_os_popen)
     func_patcher.start()
@@ -1635,7 +1625,7 @@ def patch_os_waitpid(testcase):
             result = orig_os_waitpid(pid, options)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "waitpid", autospec=True,
             side_effect=fake_os_waitpid)
     func_patcher.start()
@@ -1666,7 +1656,7 @@ def patch_os_system(testcase):
             result = orig_os_system(command)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "system", autospec=True,
             side_effect=fake_os_system)
     func_patcher.start()
@@ -1697,14 +1687,14 @@ def patch_os_spawnv(testcase):
             result = orig_os_spawnv(mode, file, args)
         return result
 
-    func_patcher = mock.patch.object(
+    func_patcher = unittest.mock.patch.object(
             os, "spawnv", autospec=True,
             side_effect=fake_os_spawnv)
     func_patcher.start()
     testcase.addCleanup(func_patcher.stop)
 
 
-# Copyright © 2015–2016 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2019 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
